@@ -1,6 +1,6 @@
 # The MIT License (MIT)
 #
-# Copyright (c) 2018-2020 Frederic Guillot
+# Copyright (c) 2018-2022 Frederic Guillot
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -21,7 +21,7 @@
 # THE SOFTWARE.
 
 import json
-from typing import List, Union, Optional, Dict
+from typing import Dict, List, Optional, Union
 
 import requests
 
@@ -42,16 +42,21 @@ class ClientError(Exception):
 class Client:
     API_VERSION = 1
 
-    def __init__(self, base_url: str,
-                 username: str = None, password: str = None,
-                 timeout: float = 30.0, api_key: str = None):
+    def __init__(
+        self,
+        base_url: str,
+        username: str = None,
+        password: str = None,
+        timeout: float = 30.0,
+        api_key: str = None,
+    ):
         self._base_url = base_url
         self._api_key = api_key
         self._username = username
         self._password = password
         self._timeout = timeout
         self._auth = (self._username, self._password) if not api_key else None
-        self._headers = {'X-Auth-Token': api_key} if api_key else None
+        self._headers = {"X-Auth-Token": api_key} if api_key else None
 
     def _get_endpoint(self, path: str) -> str:
         if len(self._base_url) > 0 and self._base_url[-1:] == "/":
@@ -75,8 +80,9 @@ class Client:
 
     def me(self) -> Dict:
         endpoint = self._get_endpoint("/me")
-        response = requests.get(endpoint, headers=self._headers,
-                                auth=self._auth, timeout=self._timeout)
+        response = requests.get(
+            endpoint, headers=self._headers, auth=self._auth, timeout=self._timeout
+        )
         if response.status_code == 200:
             return response.json()
         raise ClientError(response)
@@ -86,19 +92,22 @@ class Client:
 
     def export_feeds(self) -> str:
         endpoint = self._get_endpoint("/export")
-        response = requests.get(endpoint, headers=self._headers,
-                                auth=self._auth, timeout=self._timeout)
+        response = requests.get(
+            endpoint, headers=self._headers, auth=self._auth, timeout=self._timeout
+        )
         if response.status_code == 200:
             return response.text
         raise ClientError(response)
 
     def import_feeds(self, opml: str) -> Dict:
         endpoint = self._get_endpoint("/import")
-        response = requests.post(endpoint,
-                                 headers=self._headers,
-                                 data=opml,
-                                 auth=self._auth,
-                                 timeout=self._timeout)
+        response = requests.post(
+            endpoint,
+            headers=self._headers,
+            data=opml,
+            auth=self._auth,
+            timeout=self._timeout,
+        )
         if response.status_code == 201:
             return response.json()
         raise ClientError(response)
@@ -108,35 +117,40 @@ class Client:
         data = dict(url=website_url)
         data.update(kwargs)
 
-        response = requests.post(endpoint,
-                                 headers=self._headers,
-                                 auth=self._auth,
-                                 data=json.dumps(data),
-                                 timeout=self._timeout)
+        response = requests.post(
+            endpoint,
+            headers=self._headers,
+            auth=self._auth,
+            data=json.dumps(data),
+            timeout=self._timeout,
+        )
         if response.status_code == 200:
             return response.json()
         raise ClientError(response)
 
     def get_feeds(self) -> List[Dict]:
         endpoint = self._get_endpoint("/feeds")
-        response = requests.get(endpoint, headers=self._headers,
-                                auth=self._auth, timeout=self._timeout)
+        response = requests.get(
+            endpoint, headers=self._headers, auth=self._auth, timeout=self._timeout
+        )
         if response.status_code == 200:
             return response.json()
         raise ClientError(response)
 
     def get_feed(self, feed_id: int) -> Dict:
         endpoint = self._get_endpoint(f"/feeds/{feed_id}")
-        response = requests.get(endpoint, headers=self._headers,
-                                auth=self._auth, timeout=self._timeout)
+        response = requests.get(
+            endpoint, headers=self._headers, auth=self._auth, timeout=self._timeout
+        )
         if response.status_code == 200:
             return response.json()
         raise ClientError(response)
 
     def get_feed_icon(self, feed_id: int) -> Dict:
         endpoint = self._get_endpoint(f"/feeds/{feed_id}/icon")
-        response = requests.get(endpoint, headers=self._headers,
-                                auth=self._auth, timeout=self._timeout)
+        response = requests.get(
+            endpoint, headers=self._headers, auth=self._auth, timeout=self._timeout
+        )
         if response.status_code == 200:
             return response.json()
         raise ClientError(response)
@@ -146,11 +160,13 @@ class Client:
         data = dict(feed_url=feed_url, category_id=category_id)
         data.update(kwargs)
 
-        response = requests.post(endpoint,
-                                 headers=self._headers,
-                                 auth=self._auth,
-                                 data=json.dumps(data),
-                                 timeout=self._timeout)
+        response = requests.post(
+            endpoint,
+            headers=self._headers,
+            auth=self._auth,
+            data=json.dumps(data),
+            timeout=self._timeout,
+        )
         if response.status_code == 201:
             return response.json()["feed_id"]
         raise ClientError(response)
@@ -158,42 +174,48 @@ class Client:
     def update_feed(self, feed_id: int, **kwargs) -> Dict:
         endpoint = self._get_endpoint(f"/feeds/{feed_id}")
         data = self._get_modification_params(**kwargs)
-        response = requests.put(endpoint,
-                                headers=self._headers,
-                                auth=self._auth,
-                                data=json.dumps(data),
-                                timeout=self._timeout)
+        response = requests.put(
+            endpoint,
+            headers=self._headers,
+            auth=self._auth,
+            data=json.dumps(data),
+            timeout=self._timeout,
+        )
         if response.status_code == 201:
             return response.json()
         raise ClientError(response)
 
     def refresh_all_feeds(self) -> bool:
         endpoint = self._get_endpoint("/feeds/refresh")
-        response = requests.put(endpoint, headers=self._headers,
-                                auth=self._auth, timeout=self._timeout)
+        response = requests.put(
+            endpoint, headers=self._headers, auth=self._auth, timeout=self._timeout
+        )
         if response.status_code >= 400:
             raise ClientError(response)
         return True
 
     def refresh_feed(self, feed_id: int) -> bool:
         endpoint = self._get_endpoint(f"/feeds/{feed_id}/refresh")
-        response = requests.put(endpoint, headers=self._headers,
-                                auth=self._auth, timeout=self._timeout)
+        response = requests.put(
+            endpoint, headers=self._headers, auth=self._auth, timeout=self._timeout
+        )
         if response.status_code >= 400:
             raise ClientError(response)
         return True
 
     def delete_feed(self, feed_id: int) -> None:
         endpoint = self._get_endpoint(f"/feeds/{feed_id}")
-        response = requests.delete(endpoint, headers=self._headers,
-                                   auth=self._auth, timeout=self._timeout)
+        response = requests.delete(
+            endpoint, headers=self._headers, auth=self._auth, timeout=self._timeout
+        )
         if response.status_code != 204:
             raise ClientError(response)
 
     def get_feed_entry(self, feed_id: int, entry_id: int) -> Dict:
         endpoint = self._get_endpoint(f"/feeds/{feed_id}/entries/{entry_id}")
-        response = requests.get(endpoint, headers=self._headers,
-                                auth=self._auth, timeout=self._timeout)
+        response = requests.get(
+            endpoint, headers=self._headers, auth=self._auth, timeout=self._timeout
+        )
         if response.status_code == 200:
             return response.json()
         raise ClientError(response)
@@ -201,26 +223,30 @@ class Client:
     def get_feed_entries(self, feed_id: int, **kwargs) -> Dict:
         endpoint = self._get_endpoint(f"/feeds/{feed_id}/entries")
         params = self._get_params(**kwargs)
-        response = requests.get(endpoint,
-                                headers=self._headers,
-                                auth=self._auth,
-                                params=params,
-                                timeout=self._timeout)
+        response = requests.get(
+            endpoint,
+            headers=self._headers,
+            auth=self._auth,
+            params=params,
+            timeout=self._timeout,
+        )
         if response.status_code == 200:
             return response.json()
         raise ClientError(response)
 
     def mark_feed_entries_as_read(self, feed_id: int) -> None:
         endpoint = self._get_endpoint(f"/feeds/{feed_id}/mark-all-as-read")
-        response = requests.put(endpoint, headers=self._headers,
-                                auth=self._auth, timeout=self._timeout)
+        response = requests.put(
+            endpoint, headers=self._headers, auth=self._auth, timeout=self._timeout
+        )
         if response.status_code != 204:
             raise ClientError(response)
 
     def get_entry(self, entry_id: int) -> Dict:
         endpoint = self._get_endpoint(f"/entries/{entry_id}")
-        response = requests.get(endpoint, headers=self._headers,
-                                auth=self._auth, timeout=self._timeout)
+        response = requests.get(
+            endpoint, headers=self._headers, auth=self._auth, timeout=self._timeout
+        )
         if response.status_code == 200:
             return response.json()
         raise ClientError(response)
@@ -228,11 +254,13 @@ class Client:
     def get_entries(self, **kwargs) -> Dict:
         endpoint = self._get_endpoint("/entries")
         params = self._get_params(**kwargs)
-        response = requests.get(endpoint,
-                                headers=self._headers,
-                                auth=self._auth,
-                                params=params,
-                                timeout=self._timeout)
+        response = requests.get(
+            endpoint,
+            headers=self._headers,
+            auth=self._auth,
+            params=params,
+            timeout=self._timeout,
+        )
         if response.status_code == 200:
             return response.json()
         raise ClientError(response)
@@ -240,27 +268,40 @@ class Client:
     def update_entries(self, entry_ids: List[int], status: str) -> bool:
         endpoint = self._get_endpoint("/entries")
         data = {"entry_ids": entry_ids, "status": status}
-        response = requests.put(endpoint,
-                                headers=self._headers,
-                                auth=self._auth,
-                                data=json.dumps(data),
-                                timeout=self._timeout)
+        response = requests.put(
+            endpoint,
+            headers=self._headers,
+            auth=self._auth,
+            data=json.dumps(data),
+            timeout=self._timeout,
+        )
         if response.status_code >= 400:
             raise ClientError(response)
         return True
 
+    def fetch_entry_content(self, entry_id: int) -> Dict:
+        endpoint = self._get_endpoint(f"/entries/{entry_id}/fetch-content")
+        response = requests.get(
+            endpoint, headers=self._headers, auth=self._auth, timeout=self._timeout
+        )
+        if response.status_code == 200:
+            return response.json()
+        raise ClientError(response)
+
     def toggle_bookmark(self, entry_id: int) -> bool:
         endpoint = self._get_endpoint(f"/entries/{entry_id}/bookmark")
-        response = requests.put(endpoint, headers=self._headers,
-                                auth=self._auth, timeout=self._timeout)
+        response = requests.put(
+            endpoint, headers=self._headers, auth=self._auth, timeout=self._timeout
+        )
         if response.status_code >= 400:
             raise ClientError(response)
         return True
 
     def get_categories(self) -> List[Dict]:
         endpoint = self._get_endpoint("/categories")
-        response = requests.get(endpoint, headers=self._headers,
-                                auth=self._auth, timeout=self._timeout)
+        response = requests.get(
+            endpoint, headers=self._headers, auth=self._auth, timeout=self._timeout
+        )
         if response.status_code == 200:
             return response.json()
         raise ClientError(response)
@@ -268,11 +309,13 @@ class Client:
     def create_category(self, title: str) -> Dict:
         endpoint = self._get_endpoint("/categories")
         data = {"title": title}
-        response = requests.post(endpoint,
-                                 headers=self._headers,
-                                 auth=self._auth,
-                                 data=json.dumps(data),
-                                 timeout=self._timeout)
+        response = requests.post(
+            endpoint,
+            headers=self._headers,
+            auth=self._auth,
+            data=json.dumps(data),
+            timeout=self._timeout,
+        )
         if response.status_code == 201:
             return response.json()
         raise ClientError(response)
@@ -280,33 +323,38 @@ class Client:
     def update_category(self, category_id: int, title: str) -> Dict:
         endpoint = self._get_endpoint(f"/categories/{category_id}")
         data = {"id": category_id, "title": title}
-        response = requests.put(endpoint,
-                                headers=self._headers,
-                                auth=self._auth,
-                                data=json.dumps(data),
-                                timeout=self._timeout)
+        response = requests.put(
+            endpoint,
+            headers=self._headers,
+            auth=self._auth,
+            data=json.dumps(data),
+            timeout=self._timeout,
+        )
         if response.status_code == 201:
             return response.json()
         raise ClientError(response)
 
     def delete_category(self, category_id: int) -> None:
         endpoint = self._get_endpoint(f"/categories/{category_id}")
-        response = requests.delete(endpoint, headers=self._headers,
-                                   auth=self._auth, timeout=self._timeout)
+        response = requests.delete(
+            endpoint, headers=self._headers, auth=self._auth, timeout=self._timeout
+        )
         if response.status_code != 204:
             raise ClientError(response)
 
     def mark_category_entries_as_read(self, category_id: int) -> None:
         endpoint = self._get_endpoint(f"/categories/{category_id}/mark-all-as-read")
-        response = requests.put(endpoint, headers=self._headers,
-                                auth=self._auth, timeout=self._timeout)
+        response = requests.put(
+            endpoint, headers=self._headers, auth=self._auth, timeout=self._timeout
+        )
         if response.status_code != 204:
             raise ClientError(response)
 
     def get_users(self) -> List[Dict]:
         endpoint = self._get_endpoint("/users")
-        response = requests.get(endpoint, headers=self._headers,
-                                auth=self._auth, timeout=self._timeout)
+        response = requests.get(
+            endpoint, headers=self._headers, auth=self._auth, timeout=self._timeout
+        )
         if response.status_code == 200:
             return response.json()
         raise ClientError(response)
@@ -319,20 +367,23 @@ class Client:
 
     def _get_user(self, user_id_or_username: Union[str, int]) -> Dict:
         endpoint = self._get_endpoint(f"/users/{user_id_or_username}")
-        response = requests.get(endpoint, headers=self._headers,
-                                auth=self._auth, timeout=self._timeout)
+        response = requests.get(
+            endpoint, headers=self._headers, auth=self._auth, timeout=self._timeout
+        )
         if response.status_code == 200:
             return response.json()
         raise ClientError(response)
 
-    def create_user(self, username: str, password: str, is_admin: bool) -> Dict:
+    def create_user(self, username: str, password: str, is_admin: bool = False) -> Dict:
         endpoint = self._get_endpoint("/users")
         data = {"username": username, "password": password, "is_admin": is_admin}
-        response = requests.post(endpoint,
-                                 headers=self._headers,
-                                 auth=self._auth,
-                                 data=json.dumps(data),
-                                 timeout=self._timeout)
+        response = requests.post(
+            endpoint,
+            headers=self._headers,
+            auth=self._auth,
+            data=json.dumps(data),
+            timeout=self._timeout,
+        )
         if response.status_code == 201:
             return response.json()
         raise ClientError(response)
@@ -340,25 +391,29 @@ class Client:
     def update_user(self, user_id: int, **kwargs) -> Dict:
         endpoint = self._get_endpoint(f"/users/{user_id}")
         data = self._get_modification_params(**kwargs)
-        response = requests.put(endpoint,
-                                headers=self._headers,
-                                auth=self._auth,
-                                data=json.dumps(data),
-                                timeout=self._timeout)
+        response = requests.put(
+            endpoint,
+            headers=self._headers,
+            auth=self._auth,
+            data=json.dumps(data),
+            timeout=self._timeout,
+        )
         if response.status_code == 201:
             return response.json()
         raise ClientError(response)
 
     def delete_user(self, user_id: int) -> None:
         endpoint = self._get_endpoint(f"/users/{user_id}")
-        response = requests.delete(endpoint, headers=self._headers,
-                                   auth=self._auth, timeout=self._timeout)
+        response = requests.delete(
+            endpoint, headers=self._headers, auth=self._auth, timeout=self._timeout
+        )
         if response.status_code != 204:
             raise ClientError(response)
 
     def mark_user_entries_as_read(self, user_id: int) -> None:
         endpoint = self._get_endpoint(f"/users/{user_id}/mark-all-as-read")
-        response = requests.put(endpoint, headers=self._headers,
-                                auth=self._auth, timeout=self._timeout)
+        response = requests.put(
+            endpoint, headers=self._headers, auth=self._auth, timeout=self._timeout
+        )
         if response.status_code != 204:
             raise ClientError(response)
